/**
 * Copyright (c) 2004 Red Hat, Inc. All rights reserved.
 * This software may be freely redistributed under the terms of the
 * GNU General Public License.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Component of: Administrator GUI tool for PostgreSQL - Red Hat Edition
 */

package com.redhat.rhdb.admin;

import java.util.*;
import javax.swing.*;
import javax.swing.tree.*;
import java.util.regex.*;

import com.redhat.rhdb.admin.tree.*;
import com.redhat.rhdb.admin.pgsql.*;
import com.redhat.rhdb.admin.resources.*;
import com.redhat.rhdb.utils.*;
import com.redhat.rhdb.admin.utils.*;

/**
 * Dialog for ALTER DOMAIN ... ADD CONSTRAINT ...
 *
 * @author <a href="mailto:dbhole@redhat.com">Deepak Bhole</a>
 */

public class AlterDomainAddConstraintDialog extends AlterCreateDialog {
	
	
	public AlterDomainAddConstraintDialog(java.awt.Frame parent, DomainNode node, AdminBackendClient client) throws DialogRaiseException {
		super(parent, true);
		
		ActionFactory afactory = new ActionFactory(AdminResources.getString(AdminResources.IMAGE_PATH));
		okAction = afactory.getAction(ActionFactory.ACTION_OK);
		cancelAction = afactory.getAction(ActionFactory.ACTION_CANCEL);
		
		AdminDatabaseMetaData dbmd;
		this.backendClient = client;
		this.invocationNode = node;
		
		initComponents();
		
		setTitle(AdminResources.getString(AdminResources.DLG_ALTER_DOMAIN_ADD_CONST_TITLE, invocationNode.toString(), ""));
		
		centerDialog();
	}
	
	/** Creates new form AlterDomainAddConstraintDialog */
	public AlterDomainAddConstraintDialog(java.awt.Frame parent, boolean modal) {
		super(parent, modal);
		initComponents();
	}
	
	/** This method is called from within the constructor to
	 * initialize the form.
	 * WARNING: Do NOT modify this code. The content of this method is
	 * always regenerated by the Form Editor.
	 */
    private void initComponents() {//GEN-BEGIN:initComponents
        java.awt.GridBagConstraints gridBagConstraints;

        jlConstName = new javax.swing.JLabel();
        jtfConstName = new javax.swing.JTextField();
        jlCheckExp = new javax.swing.JLabel();
        jtfCheckExp = new javax.swing.JTextField();
        jpButtonPanel = new javax.swing.JPanel();
        jbOK = okAction.getButton();
        jbCancel = cancelAction.getButton();

        getContentPane().setLayout(new java.awt.GridBagLayout());

        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                closeDialog(evt);
            }
        });

        jlConstName.setText(AdminResources.getString(AdminResources.DLG_ALTER_DOMAIN_ADD_CONST_CONST_NAME));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(5, 5, 5, 5);
        getContentPane().add(jlConstName, gridBagConstraints);

        jtfConstName.addCaretListener(new javax.swing.event.CaretListener() {
            public void caretUpdate(javax.swing.event.CaretEvent evt) {
                jtfConstNameCaretUpdate(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(5, 0, 5, 5);
        getContentPane().add(jtfConstName, gridBagConstraints);

        jlCheckExp.setText(AdminResources.getString(AdminResources.DLG_ALTER_DOMAIN_ADD_CONST_CHECK_EXP));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(5, 5, 5, 5);
        getContentPane().add(jlCheckExp, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(5, 0, 5, 5);
        getContentPane().add(jtfCheckExp, gridBagConstraints);

        jbOK.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jbOKActionPerformed(evt);
            }
        });

        jpButtonPanel.add(jbOK);

        jbCancel.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jbCancelActionPerformed(evt);
            }
        });

        jpButtonPanel.add(jbCancel);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.EAST;
        getContentPane().add(jpButtonPanel, gridBagConstraints);

        java.awt.Dimension screenSize = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
        setBounds((screenSize.width-400)/2, (screenSize.height-125)/2, 400, 125);
    }//GEN-END:initComponents

	private void jtfConstNameCaretUpdate(javax.swing.event.CaretEvent evt) {//GEN-FIRST:event_jtfConstNameCaretUpdate
		setTitle(AdminResources.getString(AdminResources.DLG_ALTER_DOMAIN_ADD_CONST_TITLE, invocationNode.toString(), jtfConstName.getText()));
	}//GEN-LAST:event_jtfConstNameCaretUpdate

	private void jbOKActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jbOKActionPerformed
		
		String err = validateInput();
		
		if (!err.equals("")) {				
			displayError(err);
			return;
		}
		
		Hashtable parameters = new Hashtable();

		parameters.put(ObjectUpdate.PARAM_CLUSTER_NAME, invocationNode.getParentClusterName());
		parameters.put(ObjectUpdate.PARAM_DB_NAME, invocationNode.getParentDatabaseName());
		parameters.put(ObjectUpdate.PARAM_SCHEMA_NAME, invocationNode.getParentSchemaName());
		parameters.put(ObjectUpdate.PARAM_DOMAIN_NAME, invocationNode.toString());
		parameters.put(ObjectUpdate.PARAM_CONSTRAINT_NAME, jtfConstName.getText());
		parameters.put(ObjectUpdate.PARAM_CONSTRAINT, jtfCheckExp.getText());
		
		try {

			// Try to perform action. If all goes well, close dialog later

			DomainUpdate.addConstraint(backendClient, parameters);
			invocationNode.reloadView(backendClient);
			closeDialog();
		} catch (Exception e) {
			e.printStackTrace();
			displayError(e.getMessage());
		}

	}//GEN-LAST:event_jbOKActionPerformed

	/**
	 * Checks to ensure that all required fields are filled in.
	 *
	 * @return Empty string is all is ok, string with error message otherwise
	 */
	
	private String validateInput() {
		
		if (jtfConstName.getText().equals(""))
			return AdminResources.getString(AdminResources.DLG_ALTER_DOMAIN_ADD_CONST_ERR_SPECIFY_NAME);
		else if (jtfCheckExp.getText().equals(""))
			return AdminResources.getString(AdminResources.DLG_ALTER_DOMAIN_ADD_CONST_ERR_SPECIFY_CHECK_EXP);;
		
		return "";
	}
	
	private void jbCancelActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jbCancelActionPerformed
		// Close the dialog
		closeDialog();
	}//GEN-LAST:event_jbCancelActionPerformed
	
	/** Closes the dialog */
	private void closeDialog(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_closeDialog
		setVisible(false);
		dispose();
	}//GEN-LAST:event_closeDialog
	
	/**
	 * @param args the command line arguments
	 */
	public static void main(String args[]) {
		new AlterDomainAddConstraintDialog(new javax.swing.JFrame(), true).show();
	}
	
	
	private AdminBackendClient backendClient;
	private DomainNode invocationNode;
	private GenericAction okAction;
	private GenericAction cancelAction;
	
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton jbCancel;
    private javax.swing.JButton jbOK;
    private javax.swing.JLabel jlCheckExp;
    private javax.swing.JLabel jlConstName;
    private javax.swing.JPanel jpButtonPanel;
    private javax.swing.JTextField jtfCheckExp;
    private javax.swing.JTextField jtfConstName;
    // End of variables declaration//GEN-END:variables
	
}
