/**
 * Copyright (c) 2004 Red Hat, Inc. All rights reserved.
 * This software may be freely redistributed under the terms of the
 * GNU General Public License.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Component of: Administrator GUI tool for PostgreSQL - Red Hat Edition
 */

package com.redhat.rhdb.admin;

import java.util.*;
import javax.swing.*;
import javax.swing.tree.*;

import com.redhat.rhdb.admin.tree.*;
import com.redhat.rhdb.admin.pgsql.*;
import com.redhat.rhdb.admin.resources.*;
import com.redhat.rhdb.admin.utils.*;
import com.redhat.rhdb.utils.*;

/**
 * Dialog for ALTER DOMAIN (non constraint/change owner related properties)
 *
 * @author  <a href="mailto:dbhole@redhat.com">Deepak Bhole</a>
 */

public class AlterDomainDialog extends AlterCreateDialog implements java.awt.event.ActionListener {
	
	public AlterDomainDialog(java.awt.Frame parent, DomainNode node, AdminBackendClient client) throws DialogRaiseException {
		
		super(parent, true);

		ActionFactory afactory = new ActionFactory(AdminResources.getString(AdminResources.IMAGE_PATH));
        okAction = afactory.getAction(ActionFactory.ACTION_OK);
        okAction.addActionListener(this);
        cancelAction = afactory.getAction(ActionFactory.ACTION_CANCEL);
        cancelAction.addActionListener(this);
		
		initComponents();
		
		this.backendClient = client;
        this.invocationNode = node;
		
		String[][] domainInfo;
		
		// Get metaData
		try {
			dbmd = this.backendClient.getAdminDatabaseMetaData(invocationNode.getParentClusterName(), invocationNode.getParentDatabaseName());
			domainInfo = backendClient.resultSetToStringArray(dbmd.getDomainInfo(invocationNode.getOID()), new ProcessLease());
			
			if (domainInfo[0][4].charAt(0) == 't') {
				jrbSetNotNull.setSelected(true);
			} else {
				jrbDropNotNull.setSelected(true);
			}
			
			// resultSetToStringArray() is designed to provide readable data 
			// for views. While useful in dialogs too, in this case, it falls 
			// short as we need to know when typdefault is null, and the
			// function converts null to empty string. So for this purpose, we 
			// use a different function, isResultAttributeNull() which takes a 
			// result set, a row number and an attribute number and returns 
			// whether or not the slot is null
			
			if (backendClient.isResultAttributeNull(dbmd.getDomainInfo(invocationNode.getOID()), 1, 6)) {
				jrbDropDef.setSelected(true);
			} else {
				jrbSetDefTo.setSelected(true);
				jtfDefExp.setEnabled(true);
				
				String def = domainInfo[0][5];
				
				// If default starts with a single quote, it will be bound at 
				// the end by the type.. (7.4 and above only)
				
				if (def.startsWith("'")) {
					def = def.substring(1, def.indexOf("'::"));
					def = AdminMiscFunctions.getUnquotedIdentifier(def);
				}
				
				if (!AdminConfig.autoQuoteIdentifiers()) {
					def = AdminMiscFunctions.getSingleQuotedIdentifier(def);
				}
				
				jtfDefExp.setText(def);
			}
			
		} catch (Exception e) {
			// IOException is propogated from backendclient
			closeDialog();
			e.printStackTrace();
			throw new DialogRaiseException(node, e);
		}

		setTitle(AdminResources.getString(AdminResources.DLG_ALTER_DOMAIN_TITLE, invocationNode.toString()));
		
		centerDialog();
	}
	
	/** Creates new form AlterDomainDialog */
	public AlterDomainDialog(java.awt.Frame parent, boolean modal) {
		
		super(parent, true);
		
		ActionFactory afactory = new ActionFactory(AdminResources.getString(AdminResources.IMAGE_PATH));
		okAction = afactory.getAction(ActionFactory.ACTION_OK);
		okAction.addActionListener(this);
		cancelAction = afactory.getAction(ActionFactory.ACTION_CANCEL);
		cancelAction.addActionListener(this);
		
		initComponents();
	}
	
	/** This method is called from within the constructor to
	 * initialize the form.
	 * WARNING: Do NOT modify this code. The content of this method is
	 * always regenerated by the Form Editor.
	 */
    private void initComponents() {//GEN-BEGIN:initComponents
        java.awt.GridBagConstraints gridBagConstraints;

        bgDefault = new javax.swing.ButtonGroup();
        bgNotNull = new javax.swing.ButtonGroup();
        jtpProperties = new javax.swing.JTabbedPane();
        jpDefault = new javax.swing.JPanel();
        jrbSetDefTo = new javax.swing.JRadioButton();
        jtfDefExp = new javax.swing.JTextField();
        jrbDropDef = new javax.swing.JRadioButton();
        jpNotNull = new javax.swing.JPanel();
        jrbDropNotNull = new javax.swing.JRadioButton();
        jrbSetNotNull = new javax.swing.JRadioButton();
        jpButtons = new javax.swing.JPanel();
        jbOK = okAction.getButton();
        jbCancel = cancelAction.getButton();

        getContentPane().setLayout(new java.awt.GridBagLayout());

        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                closeDialog(evt);
            }
        });

        jpDefault.setLayout(new java.awt.GridBagLayout());

        jrbSetDefTo.setSelected(true);
        jrbSetDefTo.setText(AdminResources.getString(AdminResources.DLG_ALTER_DOMAIN_SET_TO));
        bgDefault.add(jrbSetDefTo);
        jrbSetDefTo.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                updateDefaultExpWidgets(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.insets = new java.awt.Insets(3, 3, 3, 3);
        jpDefault.add(jrbSetDefTo, gridBagConstraints);

        jtfDefExp.setEnabled(false);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(3, 0, 3, 3);
        jpDefault.add(jtfDefExp, gridBagConstraints);

        jrbDropDef.setText(AdminResources.getString(AdminResources.DLG_ALTER_DOMAIN_DROP_DEFAULT));
        bgDefault.add(jrbDropDef);
        jrbDropDef.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                updateDefaultExpWidgets(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(2, 3, 3, 3);
        jpDefault.add(jrbDropDef, gridBagConstraints);

        jtpProperties.addTab(AdminResources.getString(AdminResources.DLG_ALTER_DOMAIN_TAB_DEFAULT), jpDefault);

        jpNotNull.setLayout(new java.awt.GridBagLayout());

        jrbDropNotNull.setText(AdminResources.getString(AdminResources.DLG_ALTER_DOMAIN_ALLOW_NULL));
        bgNotNull.add(jrbDropNotNull);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.weightx = 1.0;
        jpNotNull.add(jrbDropNotNull, gridBagConstraints);

        jrbSetNotNull.setText(AdminResources.getString(AdminResources.DLG_ALTER_DOMAIN_DIALLOW_NULL));
        bgNotNull.add(jrbSetNotNull);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.weightx = 1.0;
        jpNotNull.add(jrbSetNotNull, gridBagConstraints);

        jtpProperties.addTab(AdminResources.getString(AdminResources.DLG_ALTER_DOMAIN_TAB_NULL), jpNotNull);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(10, 3, 3, 3);
        getContentPane().add(jtpProperties, gridBagConstraints);

        jpButtons.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.RIGHT));

        jpButtons.add(jbOK);

        jpButtons.add(jbCancel);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.EAST;
        gridBagConstraints.weightx = 1.0;
        getContentPane().add(jpButtons, gridBagConstraints);

        java.awt.Dimension screenSize = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
        setBounds((screenSize.width-500)/2, (screenSize.height-200)/2, 500, 200);
    }//GEN-END:initComponents

	private void updateDefaultExpWidgets(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_updateDefaultExpWidgets
		if (jrbSetDefTo.isSelected()) {
			jtfDefExp.setEnabled(true);
		} else {
			jtfDefExp.setEnabled(false);
		}
	}//GEN-LAST:event_updateDefaultExpWidgets

	/** Closes the dialog */
	private void closeDialog(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_closeDialog
		setVisible(false);
		dispose();
	}//GEN-LAST:event_closeDialog
	
    
    /**
     * OK/cancel button functionality
     *
     * @param e the event of clicking ok/cancel
     */
    public void actionPerformed(java.awt.event.ActionEvent e) {
        // NOTE: should only be used for ok/cancel actions.
        if (e.getActionCommand() == okAction.getActionCommand()) {
            String errorMessage = validateInput();
            if (errorMessage.equals(""))
                doOKAction();
            else
                this.displayError(errorMessage);
        } else if (e.getActionCommand() == cancelAction.getActionCommand())
            closeDialog();
    } 
	
	/**
     * Check to see if the user input makes sense.
     *
     * @return the first found error message, or an empty string if no error messages
     */
    private String validateInput() {
        return "";
    }
	
	/**
     * Performs the actual domain alteration
     */
    private void doOKAction() {
		
        Hashtable parameters = new Hashtable();

        // Add the entries into the hash table
        
        parameters.put(ObjectUpdate.PARAM_CLUSTER_NAME, invocationNode.getParentClusterName());
        parameters.put(ObjectUpdate.PARAM_DB_NAME, invocationNode.getParentDatabaseName());
        parameters.put(ObjectUpdate.PARAM_SCHEMA_NAME, invocationNode.getParentSchemaName());
        parameters.put(ObjectUpdate.PARAM_DOMAIN_NAME, invocationNode.toString());
		
		if (jrbSetDefTo.isSelected()) {
			parameters.put(ObjectUpdate.PARAM_SET_DEFAULT, ObjectUpdate.PARAM_OPT_TRUE);
		} else {
			parameters.put(ObjectUpdate.PARAM_SET_DEFAULT, ObjectUpdate.PARAM_OPT_FALSE);
		}
		
        parameters.put(ObjectUpdate.PARAM_DEFAULT, jtfDefExp.getText());
		
		if (jrbSetNotNull.isSelected()) {
			parameters.put(ObjectUpdate.PARAM_SET_NOT_NULL, ObjectUpdate.PARAM_OPT_TRUE);
		} else {
			parameters.put(ObjectUpdate.PARAM_SET_NOT_NULL, ObjectUpdate.PARAM_OPT_FALSE);
		}
		
		try {
			// Try to perform action. If all goes well, close dialog later
			DomainUpdate.alterDomain(backendClient, parameters);
			invocationNode.refreshParent(backendClient);
			closeDialog();
		} catch (Exception e) {
			e.printStackTrace();
			displayError(e.getMessage());
		}
	}
	
	/**
	 * @param args the command line arguments
	 */
	public static void main(String args[]) {
		new AlterDomainDialog(new javax.swing.JFrame(), true).show();
	}
	
	
	private AdminDatabaseMetaData dbmd;
	private AdminBackendClient backendClient;
	private DomainNode invocationNode;
	private GenericAction okAction;
	private GenericAction cancelAction;
	
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.ButtonGroup bgDefault;
    private javax.swing.ButtonGroup bgNotNull;
    private javax.swing.JButton jbCancel;
    private javax.swing.JButton jbOK;
    private javax.swing.JPanel jpButtons;
    private javax.swing.JPanel jpDefault;
    private javax.swing.JPanel jpNotNull;
    private javax.swing.JRadioButton jrbDropDef;
    private javax.swing.JRadioButton jrbDropNotNull;
    private javax.swing.JRadioButton jrbSetDefTo;
    private javax.swing.JRadioButton jrbSetNotNull;
    private javax.swing.JTextField jtfDefExp;
    private javax.swing.JTabbedPane jtpProperties;
    // End of variables declaration//GEN-END:variables
	
}
