/**
 * Copyright (c) 2004 Red Hat, Inc. All rights reserved.
 * This software may be freely redistributed under the terms of the
 * GNU General Public License.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Component of: Administrator GUI tool for PostgreSQL - Red Hat Edition
 */

package com.redhat.rhdb.admin;

import com.redhat.rhdb.admin.pgsql.*;
import com.redhat.rhdb.admin.resources.*;
import com.redhat.rhdb.admin.tree.*;
import com.redhat.rhdb.admin.utils.*;
import com.redhat.rhdb.utils.*;

/**
 *
 * Dialog for dropping domain constraints
 *
 * @author <a href="mailto:dbhole@redhat.com">Deepak Bhole</a>
 */

public class AlterDomainDropConstraintDialog extends AlterCreateDialog {
	
	/** Creates new form DropDomainConstraint */
	public AlterDomainDropConstraintDialog(java.awt.Frame parent, DomainNode node, AdminBackendClient client) throws DialogRaiseException {
		super(parent, true);
		
		this.backendClient = client;
		this.invocationNode = node;
		
		ActionFactory af = new ActionFactory();
		okAction = af.getAction(ActionFactory.ACTION_OK);
		cancelAction = af.getAction(ActionFactory.ACTION_CANCEL);
		
		initComponents();
		
		// Populate the combobox
		
		String[][] domainConstraints;
		AdminDatabaseMetaData dbmd;
		
		try {
			dbmd = this.backendClient.getAdminDatabaseMetaData(invocationNode.getParentClusterName(), invocationNode.getParentDatabaseName());
			domainConstraints = client.resultSetToStringArray(dbmd.getDomainConstraints(invocationNode.getOID()), (new ProcessLease()));
		} catch (Exception e) {
			// IOException is propogated from backendclient
			closeDialog();
			e.printStackTrace();
			throw new DialogRaiseException(node, e);
		}
		
		// Fill up the combobox
		
		for (int i=0; i < domainConstraints.length; i++) {
			acbConstraints.addItem(domainConstraints[i][0]);
		}
		
		acbConstraints.getEditor().setItem("");
		
		setTitle(AdminResources.getString(
				AdminResources.DLG_ALTER_DOMAIN_DROP_CONST_TITLE, 
				invocationNode.toString(),
				""));
		
		acbConstraints.addItemListener(new java.awt.event.ItemListener() {
			public void itemStateChanged(java.awt.event.ItemEvent evt) {
				setTitle(AdminResources.getString(
					AdminResources.DLG_ALTER_DOMAIN_DROP_CONST_TITLE, 
					invocationNode.toString(),
					(String) acbConstraints.getSelectedItem()));
			}
		});
		
		centerDialog();

	}

	/** Creates new form DropDomainConstraint */
	public AlterDomainDropConstraintDialog(java.awt.Frame parent, boolean modal) {
		super(parent, modal);
		
		ActionFactory af = new ActionFactory();
		okAction = af.getAction(ActionFactory.ACTION_OK);
		cancelAction = af.getAction(ActionFactory.ACTION_CANCEL);
		
		initComponents();
	}
	
	/** This method is called from within the constructor to
	 * initialize the form.
	 * WARNING: Do NOT modify this code. The content of this method is
	 * always regenerated by the Form Editor.
	 */
    private void initComponents() {//GEN-BEGIN:initComponents
        java.awt.GridBagConstraints gridBagConstraints;

        bgDropType = new javax.swing.ButtonGroup();
        acbConstraints = new com.redhat.rhdb.admin.AdminComboBox();
        jlConstraintToDrop = new javax.swing.JLabel();
        jpDropType = new javax.swing.JPanel();
        jrbDropRestrict = new javax.swing.JRadioButton();
        jrbDropCascade = new javax.swing.JRadioButton();
        jpButtonPanel = new javax.swing.JPanel();
        jkOK = okAction.getButton();
        jbCancel = cancelAction.getButton();

        getContentPane().setLayout(new java.awt.GridBagLayout());

        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                closeDialog(evt);
            }
        });

        acbConstraints.setEditable(true);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(5, 0, 5, 5);
        getContentPane().add(acbConstraints, gridBagConstraints);

        jlConstraintToDrop.setText(AdminResources.getString(AdminResources.DLG_ALTER_DOMAIN_DROP_CONST_CONST_TO_DROP));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.insets = new java.awt.Insets(5, 5, 5, 5);
        getContentPane().add(jlConstraintToDrop, gridBagConstraints);

        jpDropType.setLayout(new java.awt.GridBagLayout());

        jpDropType.setBorder(new javax.swing.border.TitledBorder(AdminResources.getString(AdminResources.DLG_ALTER_DOMAIN_DROP_CONST_DROP_TYPE)));
        jrbDropRestrict.setSelected(true);
        jrbDropRestrict.setText(AdminResources.getString(AdminResources.DLG_ALTER_DOMAIN_DROP_CONST_DROP_TYPE_RESTRICT));
        bgDropType.add(jrbDropRestrict);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.weightx = 1.0;
        jpDropType.add(jrbDropRestrict, gridBagConstraints);

        jrbDropCascade.setText(AdminResources.getString(AdminResources.DLG_ALTER_DOMAIN_DROP_CONST_DROP_TYPE_CASCADE));
        bgDropType.add(jrbDropCascade);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.weightx = 1.0;
        jpDropType.add(jrbDropCascade, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(5, 5, 5, 5);
        getContentPane().add(jpDropType, gridBagConstraints);

        jpButtonPanel.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.RIGHT));

        jkOK.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jkOKActionPerformed(evt);
            }
        });

        jpButtonPanel.add(jkOK);

        jbCancel.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jbCancelActionPerformed(evt);
            }
        });

        jpButtonPanel.add(jbCancel);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.EAST;
        gridBagConstraints.insets = new java.awt.Insets(1, 0, 0, 0);
        getContentPane().add(jpButtonPanel, gridBagConstraints);

        java.awt.Dimension screenSize = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
        setBounds((screenSize.width-450)/2, (screenSize.height-180)/2, 450, 180);
    }//GEN-END:initComponents

	private void jkOKActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jkOKActionPerformed
		
		String err = validateInput();
		
		if (!err.equals("")) {		
			displayError(err);
			return;
		} else {
			LAST_USER_RESP = RESP_OK;
			
			if (jrbDropRestrict.isSelected())
				DROP_TYPE = DROP_TYPE_RESTRICT;
			else
				DROP_TYPE = DROP_TYPE_CASCADE;
				
			setVisible(false);
		}
	}//GEN-LAST:event_jkOKActionPerformed

	private void jbCancelActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jbCancelActionPerformed
		setVisible(false);
	}//GEN-LAST:event_jbCancelActionPerformed
	
	/** Closes the dialog */
	private void closeDialog(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_closeDialog
		setVisible(false);
	}//GEN-LAST:event_closeDialog
	
	/**
	 * Checks to ensure that all required fields are filled in.
	 *
	 * @return Empty string is all is ok, string with error message otherwise
	 */
	
	private String validateInput() {
		if (acbConstraints.getSelectedIndex() < 0)
			return AdminResources.getString(AdminResources.DLG_ALTER_DOMAIN_DROP_CONST_ERR_SELECT_CONST_TO_DROP);
		
		return "";
	}

	/**
	 * Returns the drop type selected by the user 
	 *
	 * @return the drop type..
	 */
	
	public int getDropType () {
		return DROP_TYPE;
	}
	
	/**
	 * Returns the selected constraint name
	 *
	 * @return constraint name
	 */
	
	public String getSelectedConstraintName() {
		return (String) acbConstraints.getSelectedItem();
	}
	
	/**
	 * Gets the user reponse that caused the dialog to close (OK/Cancel)
	 *
	 * @return int indicating last response
	 */
	
	public int getUserResponse() {
		return LAST_USER_RESP;
	}
	
	/**
	 * @param args the command line arguments
	 */
	public static void main(String args[]) {
		new AlterDomainDropConstraintDialog(new javax.swing.JFrame(), true).show();
	}
	
	public static int RESP_OK = 0;
	public static int RESP_CANCEL = 1;
	
	public static int DROP_TYPE_RESTRICT = 2;
	public static int DROP_TYPE_CASCADE = 3;
	
	private int DROP_TYPE = DROP_TYPE_RESTRICT;
	private int LAST_USER_RESP = RESP_CANCEL;
	private AdminBackendClient backendClient;
	private DomainNode invocationNode;
	private GenericAction okAction;
	private GenericAction cancelAction;
	
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private com.redhat.rhdb.admin.AdminComboBox acbConstraints;
    private javax.swing.ButtonGroup bgDropType;
    private javax.swing.JButton jbCancel;
    private javax.swing.JButton jkOK;
    private javax.swing.JLabel jlConstraintToDrop;
    private javax.swing.JPanel jpButtonPanel;
    private javax.swing.JPanel jpDropType;
    private javax.swing.JRadioButton jrbDropCascade;
    private javax.swing.JRadioButton jrbDropRestrict;
    // End of variables declaration//GEN-END:variables
	
}
