/**
 * Copyright (c) 2004 Red Hat, Inc. All rights reserved.
 * This software may be freely redistributed under the terms of the
 * GNU General Public License.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Component of: Administrator GUI tool for PostgreSQL - Red Hat Edition
 */

package com.redhat.rhdb.admin;

import java.util.*;
import javax.swing.*;
import javax.swing.tree.*;

import com.redhat.rhdb.admin.tree.*;
import com.redhat.rhdb.admin.pgsql.*;
import com.redhat.rhdb.admin.resources.*;
import com.redhat.rhdb.utils.*;

/**
 * ALTER SEQUENCE dialog..
 *
 * @author  <a href="mailto:dbhole@redhat.com">Deepak Bhole</a>
 */

public class AlterSequenceDialog extends AlterCreateDialog implements java.awt.event.ActionListener {
    
    /** Creates new form SequenceDialog */
    public AlterSequenceDialog(	java.awt.Frame parent, SequenceNode node,
								AdminBackendClient client) 
								throws DialogRaiseException {
        super(parent, true);
        
        ActionFactory afactory = new ActionFactory(AdminResources.getString(AdminResources.IMAGE_PATH));
        okAction = afactory.getAction(ActionFactory.ACTION_OK);
        okAction.addActionListener(this);
        cancelAction = afactory.getAction(ActionFactory.ACTION_CANCEL);
        cancelAction.addActionListener(this);
        
        initComponents();
        
        // initialize variables
        this.backendClient = client;
        this.invocationNode = node;
        
		// Fill in the edit boxes with current values
		
		try {
			AdminDatabaseMetaData dbmd;
			String[][] sequenceMetaData;
			
			dbmd = backendClient.getAdminDatabaseMetaData(invocationNode.getParentClusterName(), invocationNode.getParentDatabaseName());
			sequenceMetaData = backendClient.resultSetToStringArray(dbmd.getSequenceInfo(invocationNode.getParentSchemaName(), invocationNode.toString()), new ProcessLease());
			
			jtfIncrement.setText(sequenceMetaData[0][3]);
			jtfMin.setText(sequenceMetaData[0][4]);
			jtfMax.setText(sequenceMetaData[0][5]);
			jtfCache.setText(sequenceMetaData[0][6]);
			
			if (sequenceMetaData[0][7].charAt(0) == 't') {
				jchCycle.setSelected(true);
			}
			
		} catch (Exception ioe) {
			closeDialog();
			ioe.printStackTrace();
			throw new DialogRaiseException(invocationNode, ioe);
		}
		
		setTitle(AdminResources.getString(AdminResources.DLG_ALTER_SEQ_TITLE, invocationNode.toString()));
        centerDialog();
    }
    
    /** Creates new form SequenceDialog */
    public AlterSequenceDialog(java.awt.Frame parent, boolean modal) {
        super(parent, modal);
        
        ActionFactory afactory = new ActionFactory(AdminResources.getString(AdminResources.IMAGE_PATH));
        okAction = afactory.getAction(ActionFactory.ACTION_OK);
        okAction.addActionListener(this);
        cancelAction = afactory.getAction(ActionFactory.ACTION_CANCEL);
        cancelAction.addActionListener(this);
        
        initComponents();
        //centerDialog();
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        java.awt.GridBagConstraints gridBagConstraints;

        bgMinValue = new javax.swing.ButtonGroup();
        bgMaxValue = new javax.swing.ButtonGroup();
        bgRestartWith = new javax.swing.ButtonGroup();
        jpParams = new javax.swing.JPanel();
        jlbIncrement = new javax.swing.JLabel();
        jtfIncrement = new javax.swing.JTextField();
        jlbCache = new javax.swing.JLabel();
        jtfCache = new javax.swing.JTextField();
        jpMinValue = new javax.swing.JPanel();
        jtfMin = new javax.swing.JTextField();
        jrbMinValue = new javax.swing.JRadioButton();
        jrbNoMinValue = new javax.swing.JRadioButton();
        jpMaxValue = new javax.swing.JPanel();
        jrbNoMaxValue = new javax.swing.JRadioButton();
        jrbMaxValue = new javax.swing.JRadioButton();
        jtfMax = new javax.swing.JTextField();
        jpRestartWith = new javax.swing.JPanel();
        jtfRestartWith = new javax.swing.JTextField();
        jrbRestartWith = new javax.swing.JRadioButton();
        jrbDontRestart = new javax.swing.JRadioButton();
        jchCycle = new javax.swing.JCheckBox();
        jpButtons = new javax.swing.JPanel();
        jbOK = okAction.getButton();
        jbCancel = cancelAction.getButton();

        getContentPane().setLayout(new java.awt.GridBagLayout());

        setTitle("LATER SEQUENCE !!!!!!!!!!! FIXME !!!!!!!!!!!!!!!");
        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                closeDialog(evt);
            }
        });

        jpParams.setLayout(new java.awt.GridBagLayout());

        jlbIncrement.setText(AdminResources.getString(AdminResources.DLG_ALTER_SEQ_INCREMENT));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.insets = new java.awt.Insets(0, 3, 6, 3);
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        jpParams.add(jlbIncrement, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 0.2;
        gridBagConstraints.insets = new java.awt.Insets(0, 0, 6, 0);
        jpParams.add(jtfIncrement, gridBagConstraints);

        jlbCache.setText(AdminResources.getString(AdminResources.DLG_ALTER_SEQ_CACHE));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 4;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(3, 3, 3, 0);
        jpParams.add(jlbCache, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 4;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 0.2;
        gridBagConstraints.insets = new java.awt.Insets(3, 0, 3, 0);
        jpParams.add(jtfCache, gridBagConstraints);

        jpMinValue.setLayout(new java.awt.GridBagLayout());

        jpMinValue.setBorder(new javax.swing.border.TitledBorder(AdminResources.getString(AdminResources.DLG_ALTER_SEQ_MIN_VAL)));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 0.2;
        jpMinValue.add(jtfMin, gridBagConstraints);

        jrbMinValue.setSelected(true);
        jrbMinValue.setText(AdminResources.getString(AdminResources.DLG_ALTER_SEQ_MIN_VAL));
        bgMinValue.add(jrbMinValue);
        jrbMinValue.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                checkMinValTF(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        jpMinValue.add(jrbMinValue, gridBagConstraints);

        jrbNoMinValue.setText(AdminResources.getString(AdminResources.DLG_ALTER_SEQ_NO_MIN_VAL));
        bgMinValue.add(jrbNoMinValue);
        jrbNoMinValue.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                checkMinValTF(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        jpMinValue.add(jrbNoMinValue, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(0, 0, 3, 0);
        jpParams.add(jpMinValue, gridBagConstraints);

        jpMaxValue.setLayout(new java.awt.GridBagLayout());

        jpMaxValue.setBorder(new javax.swing.border.TitledBorder(AdminResources.getString(AdminResources.DLG_ALTER_SEQ_MAX_VAL)));
        jrbNoMaxValue.setText(AdminResources.getString(AdminResources.DLG_ALTER_SEQ_NO_MAX_VAL));
        bgMaxValue.add(jrbNoMaxValue);
        jrbNoMaxValue.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                checkMaxValTF(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        jpMaxValue.add(jrbNoMaxValue, gridBagConstraints);

        jrbMaxValue.setSelected(true);
        jrbMaxValue.setText(AdminResources.getString(AdminResources.DLG_ALTER_SEQ_MAX_VAL));
        bgMaxValue.add(jrbMaxValue);
        jrbMaxValue.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                checkMaxValTF(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        jpMaxValue.add(jrbMaxValue, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 0.2;
        jpMaxValue.add(jtfMax, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(0, 0, 3, 0);
        jpParams.add(jpMaxValue, gridBagConstraints);

        jpRestartWith.setLayout(new java.awt.GridBagLayout());

        jpRestartWith.setBorder(new javax.swing.border.TitledBorder(AdminResources.getString(AdminResources.DLG_ALTER_SEQ_RESTART_WITH)));
        jtfRestartWith.setEnabled(false);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 0.2;
        gridBagConstraints.insets = new java.awt.Insets(0, 0, 3, 0);
        jpRestartWith.add(jtfRestartWith, gridBagConstraints);

        jrbRestartWith.setText(AdminResources.getString(AdminResources.DLG_ALTER_SEQ_RESTART_WITH));
        bgRestartWith.add(jrbRestartWith);
        jrbRestartWith.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                checkRestartWithTF(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.insets = new java.awt.Insets(0, 3, 3, 0);
        jpRestartWith.add(jrbRestartWith, gridBagConstraints);

        jrbDontRestart.setSelected(true);
        jrbDontRestart.setText(AdminResources.getString(AdminResources.DLG_ALTER_SEQ_DONT_RESTART));
        bgRestartWith.add(jrbDontRestart);
        jrbDontRestart.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                checkRestartWithTF(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.insets = new java.awt.Insets(0, 3, 3, 0);
        jpRestartWith.add(jrbDontRestart, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.insets = new java.awt.Insets(0, 0, 3, 0);
        jpParams.add(jpRestartWith, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.weighty = 0.75;
        gridBagConstraints.insets = new java.awt.Insets(5, 5, 5, 5);
        getContentPane().add(jpParams, gridBagConstraints);

        jchCycle.setText(AdminResources.getString(AdminResources.DLG_ALTER_SEQ_CYCLE));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 0.33;
        gridBagConstraints.insets = new java.awt.Insets(0, 5, 5, 0);
        getContentPane().add(jchCycle, gridBagConstraints);

        jpButtons.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.RIGHT));

        jpButtons.add(jbOK);

        jpButtons.add(jbCancel);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.insets = new java.awt.Insets(0, 5, 0, 5);
        getContentPane().add(jpButtons, gridBagConstraints);

        java.awt.Dimension screenSize = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
        setBounds((screenSize.width-550)/2, (screenSize.height-375)/2, 550, 375);
    }//GEN-END:initComponents

	private void checkRestartWithTF(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_checkRestartWithTF
		
		if (jrbRestartWith.isSelected())
			jtfRestartWith.setEnabled(true);
		else
			jtfRestartWith.setEnabled(false);
	}//GEN-LAST:event_checkRestartWithTF

	private void checkMaxValTF(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_checkMaxValTF
		
		if (jrbMaxValue.isSelected())
			jtfMax.setEnabled(true);
		else
			jtfMax.setEnabled(false);
		
	}//GEN-LAST:event_checkMaxValTF

	private void checkMinValTF(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_checkMinValTF
		
		if (jrbMinValue.isSelected())
			jtfMin.setEnabled(true);
		else
			jtfMin.setEnabled(false);
	}//GEN-LAST:event_checkMinValTF
    
    /** Closes the dialog */
    private void closeDialog(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_closeDialog
        closeDialog();
    }//GEN-LAST:event_closeDialog
    
    /**
     * @param args the command line arguments
     */
    public static void main(String args[]) {
        new AlterSequenceDialog(new javax.swing.JFrame(), true).show();
    }
    
    /**
     * Implements the ok/cancel buttons functionality
     *
     * @param e the event of clicking ok/cancel
     */
    public void actionPerformed(java.awt.event.ActionEvent e) {
        // NOTE: should only be used for ok/cancel actions.
        if (e.getActionCommand() == okAction.getActionCommand()) {
            String errorMessage = validateInput();
            if (errorMessage.equals(""))
                doOKAction();
            else
                this.displayError(errorMessage);
        } else if (e.getActionCommand() == cancelAction.getActionCommand())
            closeDialog();
    }    
    
    /**
     * Checks to see if the user input makes sense.
     *
     * @return the first found error message, or an empty string if no error messages
     */
    private String validateInput() {
		
		// Nothing to validate.. we COULD see if this is an ASC/DESC and then 
		// perform checks on min/max/etc. But it's best to let backend handle 
		// that.. one way or the other, user will have the same message.
		
        return "";
    }
    
    /**
     * Performs the actual create sequence command
     */
    private void doOKAction() {
        
        Hashtable parameters = new Hashtable();

        // Add the entries into the hash table
        
        parameters.put(ObjectUpdate.PARAM_CLUSTER_NAME, invocationNode.getParentClusterName());
        parameters.put(ObjectUpdate.PARAM_DB_NAME, invocationNode.getParentDatabaseName());
        parameters.put(ObjectUpdate.PARAM_SCHEMA_NAME, invocationNode.getParentSchemaName());
        parameters.put(ObjectUpdate.PARAM_SEQUENCE_NAME, invocationNode.toString());
        parameters.put(ObjectUpdate.PARAM_INCREMENT, jtfIncrement.getText().trim());
		
		if (jrbMinValue.isSelected())
			parameters.put(ObjectUpdate.PARAM_MINVALUE, jtfMin.getText().trim());
		else
			parameters.put(ObjectUpdate.PARAM_MINVALUE, ObjectUpdate.PARAM_OPT_NONE);
		
        if (jrbMaxValue.isSelected())
			parameters.put(ObjectUpdate.PARAM_MAXVALUE, jtfMax.getText().trim());
		else
			parameters.put(ObjectUpdate.PARAM_MAXVALUE, ObjectUpdate.PARAM_OPT_NONE);
		
		if (jrbRestartWith.isSelected())
			parameters.put(ObjectUpdate.PARAM_START, jtfRestartWith.getText().trim());
		else
			parameters.put(ObjectUpdate.PARAM_START, ObjectUpdate.PARAM_OPT_NONE);

        parameters.put(ObjectUpdate.PARAM_CACHE, jtfCache.getText().trim());
        if (jchCycle.isSelected())
            parameters.put(ObjectUpdate.PARAM_CYCLE, "true");

        try {
			
                // Try to perform action. If all goes well, close dialog later
			
                SequenceUpdate.alterSequence(backendClient, parameters);
                invocationNode.reloadView(backendClient);
                closeDialog();
        } catch (Exception e) {
                e.printStackTrace();
                displayError(e.getMessage());
        }
        
    }
	
    private AdminBackendClient backendClient;
    private SequenceNode invocationNode;
    private GenericAction okAction;
    private GenericAction cancelAction;
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.ButtonGroup bgMaxValue;
    private javax.swing.ButtonGroup bgMinValue;
    private javax.swing.ButtonGroup bgRestartWith;
    private javax.swing.JButton jbCancel;
    private javax.swing.JButton jbOK;
    private javax.swing.JCheckBox jchCycle;
    private javax.swing.JLabel jlbCache;
    private javax.swing.JLabel jlbIncrement;
    private javax.swing.JPanel jpButtons;
    private javax.swing.JPanel jpMaxValue;
    private javax.swing.JPanel jpMinValue;
    private javax.swing.JPanel jpParams;
    private javax.swing.JPanel jpRestartWith;
    private javax.swing.JRadioButton jrbDontRestart;
    private javax.swing.JRadioButton jrbMaxValue;
    private javax.swing.JRadioButton jrbMinValue;
    private javax.swing.JRadioButton jrbNoMaxValue;
    private javax.swing.JRadioButton jrbNoMinValue;
    private javax.swing.JRadioButton jrbRestartWith;
    private javax.swing.JTextField jtfCache;
    private javax.swing.JTextField jtfIncrement;
    private javax.swing.JTextField jtfMax;
    private javax.swing.JTextField jtfMin;
    private javax.swing.JTextField jtfRestartWith;
    // End of variables declaration//GEN-END:variables
    
}
