/**
 * Copyright (c) 2004 Red Hat, Inc. All rights reserved.
 * This software may be freely redistributed under the terms of the
 * GNU General Public License.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Component of: Administrator GUI tool for PostgreSQL - Red Hat Edition
 */

package com.redhat.rhdb.admin;

import java.util.*;

import com.redhat.rhdb.admin.pgsql.*;
import com.redhat.rhdb.admin.resources.*;
import com.redhat.rhdb.admin.tree.*;
import com.redhat.rhdb.admin.utils.*;
import com.redhat.rhdb.utils.*;

/**
 * Dialog for clustering indexes on a table, ALTER TABLE CLUSTER..
 *
 * @author <a href="mailto:dbhole@redhat.com">Deepak Bhole</a>
 */

public class AlterTableClusterDialog extends AlterCreateDialog {
	
	public AlterTableClusterDialog(java.awt.Frame parent, TableNode node, AdminBackendClient client) throws DialogRaiseException {
		
		super(parent, true);
		
		this.backendClient = client;
		this.invocationNode = node;
		
		ActionFactory af = new ActionFactory();
		okAction = af.getAction(ActionFactory.ACTION_OK);
		cancelAction = af.getAction(ActionFactory.ACTION_CANCEL);
		
		initComponents();
		
		acbIndex.addItemListener(new java.awt.event.ItemListener() {
			public void itemStateChanged(java.awt.event.ItemEvent evt) {
				updateTitle();
			}
		});
		
		updateIndexesAndTitle();
		centerDialog();
	}
	
	/** Creates new form ClusterTableDialog */
	public AlterTableClusterDialog(java.awt.Frame parent, boolean modal) {
		super(parent, modal);
		
		ActionFactory af = new ActionFactory();
		okAction = af.getAction(ActionFactory.ACTION_OK);
		cancelAction = af.getAction(ActionFactory.ACTION_CANCEL);
		
		initComponents();
	}
	
	/** This method is called from within the constructor to
	 * initialize the form.
	 * WARNING: Do NOT modify this code. The content of this method is
	 * always regenerated by the Form Editor.
	 */
    private void initComponents() {//GEN-BEGIN:initComponents
        java.awt.GridBagConstraints gridBagConstraints;

        jPanel1 = new javax.swing.JPanel();
        jlIndex = new javax.swing.JLabel();
        acbIndex = new com.redhat.rhdb.admin.AdminComboBox();
        jbOK = okAction.getButton();
        jbCancel = cancelAction.getButton();

        getContentPane().setLayout(new java.awt.GridBagLayout());

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        setTitle("");
        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                closeDialog(evt);
            }
        });

        jPanel1.setLayout(new java.awt.GridBagLayout());

        jPanel1.setMinimumSize(new java.awt.Dimension(148, 48));
        jPanel1.setPreferredSize(new java.awt.Dimension(400, 60));
        jlIndex.setText(AdminResources.getString(AdminResources.DLG_ALTER_TABLE_CLUSTER_INDEX));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(0, 3, 0, 3);
        jPanel1.add(jlIndex, gridBagConstraints);

        acbIndex.setEditable(true);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.gridwidth = 3;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weightx = 1.0;
        jPanel1.add(acbIndex, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weightx = 1.0;
        getContentPane().add(jPanel1, gridBagConstraints);

        jbOK.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jbOKActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.SOUTHEAST;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(0, 0, 2, 77);
        getContentPane().add(jbOK, gridBagConstraints);

        jbCancel.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jbCancelActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.SOUTHEAST;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(0, 0, 2, 2);
        getContentPane().add(jbCancel, gridBagConstraints);

        pack();
    }//GEN-END:initComponents

	private void jbOKActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jbOKActionPerformed
		
		String err = validateInput();

		if (!err.equals("")) {
			displayError(err);
			return;
		}
				
		Hashtable parameters = new Hashtable();

		parameters.put(ObjectUpdate.PARAM_CLUSTER_NAME, invocationNode.getParentClusterName());
		parameters.put(ObjectUpdate.PARAM_DB_NAME, invocationNode.getParentDatabaseName());
		parameters.put(ObjectUpdate.PARAM_SCHEMA_NAME, invocationNode.getParentSchemaName());
		parameters.put(ObjectUpdate.PARAM_TABLE_NAME, invocationNode.toString());
		parameters.put(ObjectUpdate.PARAM_INDEX_NAME, acbIndex.getEditor().getItem());

		try {

			// Try to perform action. If all goes well, close dialog later

			TableUpdate.markForCluster(backendClient, parameters);
			closeDialog();
		} catch (Exception e) {
			e.printStackTrace();
			displayError(e.getMessage());
		}
		
	}//GEN-LAST:event_jbOKActionPerformed

	private void jbCancelActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jbCancelActionPerformed
		// Close the dialog
		
		closeDialog();
		
	}//GEN-LAST:event_jbCancelActionPerformed
    
    /** Closes the dialog */
    private void closeDialog(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_closeDialog
        setVisible(false);
        dispose();
    }//GEN-LAST:event_closeDialog
    
	/**
	 * Overloaded method. Makes sure that "update" happens iff this is a selection event
	 */
	
	private void updateIndexesAndTitle(java.awt.event.ItemEvent evt) {
		if (evt.getStateChange() == java.awt.event.ItemEvent.SELECTED)
			updateIndexesAndTitle();
	}

	private void updateIndexesAndTitle() {
		
		String[][] indexNames;

		try {
			AdminDatabaseMetaData dbmd = backendClient.getAdminDatabaseMetaData(invocationNode.getParentClusterName(), invocationNode.getParentDatabaseName());
			indexNames = backendClient.resultSetToStringArray(dbmd.getIndexes(invocationNode.getParentSchemaName(),invocationNode.toString()), new ProcessLease());
		} catch (Exception e) {
			e.printStackTrace();
			MessageBox.error(this, "", e.getMessage());
			return;
		}

		acbIndex.removeAllItems();
			
		for (int i=0; i < indexNames.length; i++) {
			acbIndex.addItem(indexNames[i][0]);
		}
		
		updateTitle();
	}
	
	private void updateTitle() {
		
		String tableName, selectedIndex;
		
		selectedIndex = acbIndex.getSelectedItem() != null ? (String) acbIndex.getSelectedItem() : "";
		
		setTitle(AdminResources.getString(AdminResources.DLG_ALTER_TABLE_CLUSTER_TITLE, invocationNode.toString(), selectedIndex));
	}
	
	/**
	 * Validates the input
	 *
	 * @return String containing error messages, if any.
	 */
	
	private String validateInput() {

		if (((String) acbIndex.getEditor().getItem()).equals(""))
			return AdminResources.getString(AdminResources.DLG_ALTER_TABLE_CLUSTER_ERR_PLEASE_SELECT_INDEX);

		return "";
	}

    /**
     * @param args the command line arguments
     */
    public static void main(String args[]) {
        new ClusterTableDialog(new javax.swing.JFrame(), true).show();
    }
    
	
	private AdminBackendClient backendClient;
	private TableNode invocationNode;
	private GenericAction okAction;
	private GenericAction cancelAction;
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private com.redhat.rhdb.admin.AdminComboBox acbIndex;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JButton jbCancel;
    private javax.swing.JButton jbOK;
    private javax.swing.JLabel jlIndex;
    // End of variables declaration//GEN-END:variables
    
}
