/**
 * Copyright (c) 2004 Red Hat, Inc. All rights reserved.
 * This software may be freely redistributed under the terms of the
 * GNU General Public License.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Component of: Administrator GUI tool for PostgreSQL - Red Hat Edition
 */

package com.redhat.rhdb.admin;

import java.util.*;

import com.redhat.rhdb.admin.pgsql.*;
import com.redhat.rhdb.admin.resources.*;
import com.redhat.rhdb.admin.tree.*;
import com.redhat.rhdb.utils.*;

/**
 * Dialog for ALTER TABLE SET WITHOUT OIDS...
 *
 * @author  <a href="mailto: dbhole@redhat.com">Deepak Bhole</a>
 */

public class AlterTableSetWithoutOIDSDialog extends AlterCreateDialog {
	
	public AlterTableSetWithoutOIDSDialog(java.awt.Frame parent, TableNode node, AdminBackendClient client) throws DialogRaiseException {
		
		super(parent, true);
		
		this.backendClient = client;
		this.invocationNode = node;
		
		ActionFactory af = new ActionFactory();
		okAction = af.getAction(ActionFactory.ACTION_OK);
		cancelAction = af.getAction(ActionFactory.ACTION_CANCEL);
		
		setTitle(AdminResources.getString(AdminResources.DLG_ALTER_TABLE_SET_WO_OIDS_TITLE, invocationNode.toString()));
		
		initComponents();
		
		try {
			AdminDatabaseMetaData dbmd = this.backendClient.getAdminDatabaseMetaData(invocationNode.getParentClusterName(), invocationNode.getParentDatabaseName());

			if (dbmd.isSQLInheritenceOn())
				jrbThisTabAndDesc.setSelected(true);
			else
				jrbThisTabOnly.setSelected(true);

		} catch (Exception e) {
			
			// IOException is propogated from backendclient
			
			closeDialog();
			e.printStackTrace();
			throw new DialogRaiseException(node, e);
			
		}
		
		centerDialog();
	}
	
	/** Creates new form ClusterTableDialog */
	public AlterTableSetWithoutOIDSDialog(java.awt.Frame parent, boolean modal) {
		super(parent, modal);
		
		ActionFactory af = new ActionFactory();
		okAction = af.getAction(ActionFactory.ACTION_OK);
		cancelAction = af.getAction(ActionFactory.ACTION_CANCEL);
		
		initComponents();
	}
	
	/** This method is called from within the constructor to
	 * initialize the form.
	 * WARNING: Do NOT modify this code. The content of this method is
	 * always regenerated by the Form Editor.
	 */
    private void initComponents() {//GEN-BEGIN:initComponents
        java.awt.GridBagConstraints gridBagConstraints;

        bgOnly = new javax.swing.ButtonGroup();
        jpInhPanel = new javax.swing.JPanel();
        jrbThisTabOnly = new javax.swing.JRadioButton();
        jrbThisTabAndDesc = new javax.swing.JRadioButton();
        jpButtonPanel = new javax.swing.JPanel();
        jbOK = okAction.getButton();
        jbCancel = cancelAction.getButton();

        getContentPane().setLayout(new java.awt.GridBagLayout());

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        setTitle("");
        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                closeDialog(evt);
            }
        });

        jpInhPanel.setLayout(new java.awt.GridBagLayout());

        jpInhPanel.setMinimumSize(new java.awt.Dimension(148, 48));
        jpInhPanel.setPreferredSize(new java.awt.Dimension(400, 60));
        jrbThisTabOnly.setText(AdminResources.getString(AdminResources.DLG_ALTER_TABLE_SET_WO_OIDS_THIS_TAB_ONLY));
        bgOnly.add(jrbThisTabOnly);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        jpInhPanel.add(jrbThisTabOnly, gridBagConstraints);

        jrbThisTabAndDesc.setText(AdminResources.getString(AdminResources.DLG_ALTER_TABLE_SET_WO_OIDS_THIS_TAB_AND_DESC));
        bgOnly.add(jrbThisTabAndDesc);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        jpInhPanel.add(jrbThisTabAndDesc, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weightx = 1.0;
        getContentPane().add(jpInhPanel, gridBagConstraints);

        jpButtonPanel.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.RIGHT));

        jbOK.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jbOKActionPerformed(evt);
            }
        });

        jpButtonPanel.add(jbOK);

        jbCancel.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jbCancelActionPerformed(evt);
            }
        });

        jpButtonPanel.add(jbCancel);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.EAST;
        gridBagConstraints.weightx = 1.0;
        getContentPane().add(jpButtonPanel, gridBagConstraints);

        pack();
    }//GEN-END:initComponents

	private void jbCancelActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jbCancelActionPerformed
		// Close the dialog
		
		closeDialog();
	}//GEN-LAST:event_jbCancelActionPerformed

	private void jbOKActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jbOKActionPerformed

		String err = validateInput();
		
		if (!err.equals("")) {
			displayError(err);
			return;
		}
		
		boolean only = false;
		
		if (this.jrbThisTabOnly.isSelected())
			only = true;
		
		try {
			
			// Try to perform action. If all goes well, close dialog later
			
			TableUpdate.setWithoutOID(backendClient, invocationNode, only);
			closeDialog();
		} catch (Exception e) {
			e.printStackTrace();
			displayError(e.getMessage());
		}
		
	}//GEN-LAST:event_jbOKActionPerformed
    
    /** Closes the dialog */
    private void closeDialog(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_closeDialog
        setVisible(false);
        dispose();
    }//GEN-LAST:event_closeDialog
    
	
	/**
	 * Validates the input
	 *
	 * @return String containing error messages, if any.
	 */
	
	private String validateInput() {
		return "";
	}

    /**
     * @param args the command line arguments
     */
    public static void main(String args[]) {
        new AlterTableSetWithoutOIDSDialog(new javax.swing.JFrame(), true).show();
    }
    
	
	private AdminBackendClient backendClient;
	private TableNode invocationNode;
	private GenericAction okAction;
	private GenericAction cancelAction;
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.ButtonGroup bgOnly;
    private javax.swing.JButton jbCancel;
    private javax.swing.JButton jbOK;
    private javax.swing.JPanel jpButtonPanel;
    private javax.swing.JPanel jpInhPanel;
    private javax.swing.JRadioButton jrbThisTabAndDesc;
    private javax.swing.JRadioButton jrbThisTabOnly;
    // End of variables declaration//GEN-END:variables
    
}
