/**
 * Copyright (c) 2004 Red Hat, Inc. All rights reserved.
 * This software may be freely redistributed under the terms of the
 * GNU General Public License.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Component of: Administrator GUI tool for PostgreSQL - Red Hat Edition
 */

package com.redhat.rhdb.admin;

import com.redhat.rhdb.admin.tree.*;
import com.redhat.rhdb.admin.pgsql.*;
import com.redhat.rhdb.admin.resources.*;
import com.redhat.rhdb.utils.*;

/**
 * Dialog for dropping columns. ALTER TABLE DROP COLUMN ...
 *
 * @author <a href="mailto:dbhole@redhat.com">Deepak Bhole</a>
 */

public class DropColumnDialog extends AlterCreateDialog {
	
	public DropColumnDialog(java.awt.Frame parent, AdminBackendClient backendClient, ColumnUnderTableNode invocationNode, String colName) throws DialogRaiseException {
		super(parent, true);
		
		ActionFactory afactory = new ActionFactory(AdminResources.getString(AdminResources.IMAGE_PATH));
		okAction = afactory.getAction(ActionFactory.ACTION_OK);
		cancelAction = afactory.getAction(ActionFactory.ACTION_CANCEL);
		
		initComponents();
		
		try {
			
			AdminDatabaseMetaData dbmd = backendClient.getAdminDatabaseMetaData(invocationNode.getParentClusterName(), invocationNode.getParentDatabaseName());
						
			if (dbmd.isSQLInheritenceOn())
				jrbThisTabAndDesc.setSelected(true);
			else
				jrbThisTabOnly.setSelected(true);

		} catch (Exception e) {

			closeDialog();
			e.printStackTrace();
			throw new DialogRaiseException(invocationNode, e);
			
		}
		
		setTitle(AdminResources.getString(AdminResources.DLG_DROP_COLUMN_TITLE, colName));
	}
	
	/** Creates new form DropColumnDialog */
	public DropColumnDialog(java.awt.Frame parent, boolean modal) {
		super(parent, modal);
		
		ActionFactory afactory = new ActionFactory(AdminResources.getString(AdminResources.IMAGE_PATH));
		okAction = afactory.getAction(ActionFactory.ACTION_OK);
		cancelAction = afactory.getAction(ActionFactory.ACTION_CANCEL);
		
		initComponents();
	}
	
	/** This method is called from within the constructor to
	 * initialize the form.
	 * WARNING: Do NOT modify this code. The content of this method is
	 * always regenerated by the Form Editor.
	 */
    private void initComponents() {//GEN-BEGIN:initComponents
        java.awt.GridBagConstraints gridBagConstraints;

        bgInhOpt = new javax.swing.ButtonGroup();
        bgDropType = new javax.swing.ButtonGroup();
        jpInhOptions = new javax.swing.JPanel();
        jrbThisTabOnly = new javax.swing.JRadioButton();
        jrbThisTabAndDesc = new javax.swing.JRadioButton();
        jpDropType = new javax.swing.JPanel();
        jrbRestrict = new javax.swing.JRadioButton();
        jrbCascade = new javax.swing.JRadioButton();
        jpButtonPanel = new javax.swing.JPanel();
        jbOK = okAction.getButton();
        jbCancel = cancelAction.getButton();

        getContentPane().setLayout(new java.awt.GridBagLayout());

        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                closeDialog(evt);
            }
        });

        jpInhOptions.setLayout(new java.awt.BorderLayout());

        jpInhOptions.setBorder(new javax.swing.border.TitledBorder(AdminResources.getString(AdminResources.DLG_DROP_COLUMN_INH_OPTS)));
        jrbThisTabOnly.setText(AdminResources.getString(AdminResources.DLG_DROP_COLUMN_THIS_TAB_ONLY));
        bgInhOpt.add(jrbThisTabOnly);
        jpInhOptions.add(jrbThisTabOnly, java.awt.BorderLayout.NORTH);

        jrbThisTabAndDesc.setText(AdminResources.getString(AdminResources.DLG_DROP_COLUMN_THIS_TAB_AND_DESC));
        bgInhOpt.add(jrbThisTabAndDesc);
        jpInhOptions.add(jrbThisTabAndDesc, java.awt.BorderLayout.CENTER);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(5, 5, 0, 5);
        getContentPane().add(jpInhOptions, gridBagConstraints);

        jpDropType.setLayout(new java.awt.BorderLayout());

        jpDropType.setBorder(new javax.swing.border.TitledBorder(AdminResources.getString(AdminResources.DLG_DROP_COLUMN_DROP_TYPE)));
        jrbRestrict.setSelected(true);
        jrbRestrict.setText(AdminResources.getString(AdminResources.DLG_DROP_COLUMN_DROP_TYPE_RESTRICT));
        bgDropType.add(jrbRestrict);
        jpDropType.add(jrbRestrict, java.awt.BorderLayout.NORTH);

        jrbCascade.setText(AdminResources.getString(AdminResources.DLG_DROP_COLUMN_DROP_TYPE_CASCADE));
        bgDropType.add(jrbCascade);
        jpDropType.add(jrbCascade, java.awt.BorderLayout.CENTER);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(5, 5, 0, 5);
        getContentPane().add(jpDropType, gridBagConstraints);

        jpButtonPanel.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.RIGHT));

        jbOK.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jbOKActionPerformed(evt);
            }
        });

        jpButtonPanel.add(jbOK);

        jbCancel.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jbCancelActionPerformed(evt);
            }
        });

        jpButtonPanel.add(jbCancel);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(5, 5, 5, 0);
        getContentPane().add(jpButtonPanel, gridBagConstraints);

        java.awt.Dimension screenSize = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
        setBounds((screenSize.width-400)/2, (screenSize.height-210)/2, 400, 210);
    }//GEN-END:initComponents

	private void jbOKActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jbOKActionPerformed
		
		if (jrbRestrict.isSelected())
			DROP_TYPE = DROP_TYPE_RESTRICT;
		else
			DROP_TYPE = DROP_TYPE_CASCADE;
		
		if (jrbThisTabOnly.isSelected())
			INH_OPT = INH_OPT_THIS_TAB_ONLY;
		else
			INH_OPT = INH_OPT_THIS_TAB_AND_DESC;
		
		LAST_USER_RESP = RESP_OK;

		setVisible(false);
	}//GEN-LAST:event_jbOKActionPerformed

	private void jbCancelActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jbCancelActionPerformed
		setVisible(false);
	}//GEN-LAST:event_jbCancelActionPerformed
	
	/** Closes the dialog */
	private void closeDialog(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_closeDialog
		setVisible(false);
	}//GEN-LAST:event_closeDialog
	
	/**
	 * Returns the drop type selected by the user 
	 *
	 * @return the drop type..
	 */
	
	public int getDropType () {
		return DROP_TYPE;
	}
	
	/**
	 * Returns the inheritence option for this drop
	 *
	 * @return the inheritence option
	 */
	
	public int getInhOption() {
		return INH_OPT;
	}
	
	/**
	 * Gets the user reponse that caused the dialog to close (OK/Cancel)
	 *
	 * @return int indicating last response
	 */
	
	public int getUserResponse() {
		return LAST_USER_RESP;
	}
	
	/**
	 * @param args the command line arguments
	 */
	public static void main(String args[]) {
		new DropColumnDialog(new javax.swing.JFrame(), true).show();
	}

	public static int RESP_OK = 0;
	public static int RESP_CANCEL = 1;
	
	public static int DROP_TYPE_RESTRICT = 2;
	public static int DROP_TYPE_CASCADE = 3;
	
	public static int INH_OPT_THIS_TAB_ONLY = 4;
	public static int INH_OPT_THIS_TAB_AND_DESC = 5;
	
	private int INH_OPT = INH_OPT_THIS_TAB_ONLY;
	private int DROP_TYPE = DROP_TYPE_RESTRICT;
	private int LAST_USER_RESP = RESP_CANCEL;

	private GenericAction okAction;
	private GenericAction cancelAction;
	
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.ButtonGroup bgDropType;
    private javax.swing.ButtonGroup bgInhOpt;
    private javax.swing.JButton jbCancel;
    private javax.swing.JButton jbOK;
    private javax.swing.JPanel jpButtonPanel;
    private javax.swing.JPanel jpDropType;
    private javax.swing.JPanel jpInhOptions;
    private javax.swing.JRadioButton jrbCascade;
    private javax.swing.JRadioButton jrbRestrict;
    private javax.swing.JRadioButton jrbThisTabAndDesc;
    private javax.swing.JRadioButton jrbThisTabOnly;
    // End of variables declaration//GEN-END:variables
	
}
