/**
 * Copyright (c) 2004 Red Hat, Inc. All rights reserved.
 * This software may be freely redistributed under the terms of the
 * GNU General Public License.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Component of: Administrator GUI tool for PostgreSQL - Red Hat Edition
 */

package com.redhat.rhdb.admin;

import java.util.*;
import javax.swing.*;
import javax.swing.tree.*;

import com.redhat.rhdb.admin.tree.*;
import com.redhat.rhdb.admin.pgsql.*;
import com.redhat.rhdb.admin.resources.*;
import com.redhat.rhdb.admin.utils.*;
import com.redhat.rhdb.utils.*;

/**
 * Dialog for renaming columns. ALTER TABLE RENAME COLUMN ...
 *
 * @author <a href="mailto:dbhole@redhat.com">Deepak Bhole</a>
 */


public class RenameColumnDialog extends AlterCreateDialog {
	
	/** Creates new form RenameColumnDialog */
	public RenameColumnDialog(java.awt.Frame parent, ColumnUnderTableNode node, AdminBackendClient client) throws DialogRaiseException {

		super(parent, true);
		
		ActionFactory afactory = new ActionFactory(AdminResources.getString(AdminResources.IMAGE_PATH));
        okAction = afactory.getAction(ActionFactory.ACTION_OK);
        cancelAction = afactory.getAction(ActionFactory.ACTION_CANCEL);
		
		this.backendClient = client;
		this.invocationNode = node;
		
		initComponents();
		
		if (AdminConfig.autoQuoteIdentifiers())
			jtfName.setText(invocationNode.toString());
		else
			jtfName.setText(AdminMiscFunctions.getSingleQuotedIdentifier(invocationNode.toString()));
		
		try {
			
			AdminDatabaseMetaData dbmd = backendClient.getAdminDatabaseMetaData(invocationNode.getParentClusterName(), invocationNode.getParentDatabaseName());
						
			if (dbmd.isSQLInheritenceOn())
				jrbThisTabAndDesc.setSelected(true);
			else
				jrbThisTabOnly.setSelected(true);

		} catch (Exception e) {

			closeDialog();
			e.printStackTrace();
			throw new DialogRaiseException(invocationNode, e);
			
		}

		centerDialog();
	}
	
	/** Creates new form RenameColumnDialog */
	public RenameColumnDialog(java.awt.Frame parent, boolean modal) {
		super(parent, modal);
		
		ActionFactory afactory = new ActionFactory(AdminResources.getString(AdminResources.IMAGE_PATH));
        okAction = afactory.getAction(ActionFactory.ACTION_OK);
        cancelAction = afactory.getAction(ActionFactory.ACTION_CANCEL);
		
		initComponents();
	}
	
	/** This method is called from within the constructor to
	 * initialize the form.
	 * WARNING: Do NOT modify this code. The content of this method is
	 * always regenerated by the Form Editor.
	 */
    private void initComponents() {//GEN-BEGIN:initComponents
        java.awt.GridBagConstraints gridBagConstraints;

        bgInhOpt = new javax.swing.ButtonGroup();
        jtfName = new javax.swing.JTextField();
        jlbName = new javax.swing.JLabel();
        jpInhOptions = new javax.swing.JPanel();
        jrbThisTabOnly = new javax.swing.JRadioButton();
        jrbThisTabAndDesc = new javax.swing.JRadioButton();
        jpButtonPanel = new javax.swing.JPanel();
        jbOK = okAction.getButton();
        jbCancel = cancelAction.getButton();

        getContentPane().setLayout(new java.awt.GridBagLayout());

        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                closeDialog(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.insets = new java.awt.Insets(6, 0, 0, 6);
        getContentPane().add(jtfName, gridBagConstraints);

        jlbName.setText(AdminResources.getString(AdminResources.DLG_RENAME_COLUMN_NAME));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.insets = new java.awt.Insets(6, 3, 0, 0);
        getContentPane().add(jlbName, gridBagConstraints);

        jpInhOptions.setLayout(new java.awt.BorderLayout());

        jpInhOptions.setBorder(new javax.swing.border.TitledBorder(AdminResources.getString(AdminResources.DLG_RENAME_COLUMN_INH_OPTS)));
        jrbThisTabOnly.setText(AdminResources.getString(AdminResources.DLG_RENAME_COLUMN_THIS_TAB_ONLY));
        bgInhOpt.add(jrbThisTabOnly);
        jpInhOptions.add(jrbThisTabOnly, java.awt.BorderLayout.NORTH);

        jrbThisTabAndDesc.setText(AdminResources.getString(AdminResources.DLG_RENAME_COLUMN_THIS_TAB_AND_DESC));
        bgInhOpt.add(jrbThisTabAndDesc);
        jpInhOptions.add(jrbThisTabAndDesc, java.awt.BorderLayout.CENTER);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.insets = new java.awt.Insets(5, 5, 0, 5);
        gridBagConstraints.weightx = 1.0;
        getContentPane().add(jpInhOptions, gridBagConstraints);

        jpButtonPanel.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.RIGHT));

        jbOK.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jbOKActionPerformed(evt);
            }
        });

        jpButtonPanel.add(jbOK);

        jbCancel.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jbCancelActionPerformed(evt);
            }
        });

        jpButtonPanel.add(jbCancel);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.insets = new java.awt.Insets(3, 3, 3, 3);
        getContentPane().add(jpButtonPanel, gridBagConstraints);

        java.awt.Dimension screenSize = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
        setBounds((screenSize.width-400)/2, (screenSize.height-175)/2, 400, 175);
    }//GEN-END:initComponents

	private void jbOKActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jbOKActionPerformed
		
		/* Barring some minor changes, this function is similar to the function 
		 * executed on OK in rename dialogs.. cmost omments have therefore been 
		 * ommitted.
		 */

		String err = validateInput();
		
		if (!err.equals("")) {
			displayError(err);
			return;
		}
		
		try {
			
			String quotedName, newNodeName, inhOpt;
			TreePath parentPath;
			
			boolean wasSelectedBefore = invocationNode.isSelected();
			boolean wasDescendantOfSelectedNode = invocationNode.getPath().isDescendant(invocationNode.getParentTree().getSelectionPath());
			
			if (AdminConfig.autoQuoteIdentifiers())
				quotedName = AdminMiscFunctions.getDoubleQuotedIdentifier(jtfName.getText());
			else
				quotedName = jtfName.getText();
			
			if (AdminConfig.autoQuoteIdentifiers()) {
				newNodeName = jtfName.getText();
			} else {
				
				// If in quotes, we need to extract what is inside. No need to turn
				// it to lowercase then.
				
				newNodeName = jtfName.getText();
				
				// Starts with \" => Ends with \" else backend will give
				// parse error..
				
				if (newNodeName.startsWith("\"")) {
					newNodeName = newNodeName.substring(1, (newNodeName.length() - 1));
				} else {
					newNodeName = newNodeName.toLowerCase();
				}
			}
			
			if (jrbThisTabOnly.isSelected())
				inhOpt = ObjectUpdate.PARAM_OPT_INH_ONLY;
			else
				inhOpt = ObjectUpdate.PARAM_OPT_INH_DESC;
			
			ColumnUpdate.renameTableColumn(backendClient, invocationNode, quotedName, inhOpt);
			
			parentPath = ((AdminTreeNode) invocationNode.getParent()).getPath();
			
			if (wasSelectedBefore || wasDescendantOfSelectedNode)
				invocationNode.refreshParent(backendClient, parentPath.pathByAddingChild(newNodeName));
			else
				invocationNode.refreshParent(backendClient);
			
			closeDialog();
		} catch (ObjectUpdateException renameException) {
			renameException.printStackTrace();
			displayError(renameException.getMessage());
		} catch (com.redhat.rhdb.admin.tree.TreeNodePopulationException refreshException) {
			refreshException.printStackTrace();
			displayError(refreshException.getMessage());
		}
		
	}//GEN-LAST:event_jbOKActionPerformed

	private void jbCancelActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jbCancelActionPerformed
		closeDialog();
	}//GEN-LAST:event_jbCancelActionPerformed
	
	/** Closes the dialog */
	private void closeDialog(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_closeDialog
		setVisible(false);
		dispose();
	}//GEN-LAST:event_closeDialog
	
	/**
     * Check to see if the user input makes sense.
     *
     * @return the first found error message, or an empty string if no error messages
     */
    private String validateInput() {
        
        // check for name
        if (jtfName.getText().trim().equals("") || jtfName.getText().trim().equals("\"\"")) 
            return AdminResources.getString(AdminResources.DLG_RENAME_COLUMN_ERR_NO_NAME, invocationNode.getNodeType());
        
        return "";
    }
	
	/**
	 * @param args the command line arguments
	 */
	public static void main(String args[]) {
		new RenameColumnDialog(new javax.swing.JFrame(), true).show();
	}
	
	private AdminBackendClient backendClient;
	private ColumnUnderTableNode invocationNode;
	GenericAction okAction, cancelAction;
	
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.ButtonGroup bgInhOpt;
    private javax.swing.JButton jbCancel;
    private javax.swing.JButton jbOK;
    private javax.swing.JLabel jlbName;
    private javax.swing.JPanel jpButtonPanel;
    private javax.swing.JPanel jpInhOptions;
    private javax.swing.JRadioButton jrbThisTabAndDesc;
    private javax.swing.JRadioButton jrbThisTabOnly;
    private javax.swing.JTextField jtfName;
    // End of variables declaration//GEN-END:variables
	
}
